/**
 * Created by TonyJiang on 16/5/17.
 */

import React from 'react';

//工具
import {propTypes , defaultProps} from '../common/props';

//子组件
import {STATUS , TYPE , DIRECTION , SIDE} from '../common/constant';
import Refresher from '../common/Refresher';
import ScrollerBase from './ScrollerBase';


class WrapperForNative extends React.Component{

    static propTypes = propTypes;

    static defaultProps = defaultProps;

    state = {
        reach : {
            [SIDE.TOP] : false,
            [SIDE.BOTTOM] : false
        },

        enableRefresh: false,
        enableLoadMore: false,


        //使用window监听事件后, 需要判断当前是否处于活跃状态
        isPress: false,


        //在_initDynamicState中定义
        status : undefined,
        isReach : undefined,
        springConstant : undefined
    };

    _stateInShadow = {
        status : undefined
    };

    //根据props中事件监听, 判断是否启用refresh和loadMore
    componentWillMount(){
        this._checkEnable(this.props);
    }

    //根据props中事件监听, 判断是否启用refresh和loadMore
    componentWillReceiveProps(newProps){
        this._checkEnable(newProps);
    }

    componentDidMount(){
        this._initDynamicState();
        this._initEvent();
    }

    _checkEnable(props){
        this.state.enableRefresh = props.hasOwnProperty('refreshHandler') && (typeof props.refreshHandler) == 'function' ;
        this.state.enableLoadMore = props.hasOwnProperty('moreHandler') && (typeof props.moreHandler) == 'function';
    }

    _checkInComponents(parent: HTMLElement, child: HTMLElement){
        var dom = child;
        while(dom !== null){
            if(dom == parent){
                return true;
            }else{
                dom = dom.parentNode;
            }
        }
        return false;
    }


    _getStyles() {

        const {displayWidth, displayHeight , style, refreshColor, moreColor, contentBackgroundColor, rootBackgroundColor} = this.props;

        const styles = {

            root: Object.assign({
                position: 'relative',
                width: displayWidth,
                height : displayHeight,
                overflowY:'hidden',
                background: rootBackgroundColor
            },style || {}),
            
            refresh : {
                color: refreshColor
            },
            
            more: {
                color: moreColor
            },


            scrollerWrapper : {
                position : 'absolute',
                top:0,
                left:0,
                right:0,
                bottom:0,
                zIndex:2,
                overflow:'hidden',
                background: contentBackgroundColor
            },

            contentWrapper : {
                minHeight : mutants.env.os == mutants.env.constant.os.ios ? '100.2%' : '100%'
            }

        };

        return styles;

    }

    /**
     * 定义动态state
     * 为了解决组件状态发生改变时,不想触发自己的render,但是想要改变子组件的状态.
     * 通过PropertySetter来调用子组件改变对应状态的方法.
     */
    _initDynamicState(){
        //定义state.status
        Object.defineProperty(this.state , 'status' , {
            get : () => {
                return this._stateInShadow.status;
            },
            set : (value) => {
                if(this._stateInShadow.status != value){
                    this._stateInShadow.status = value;

                    //根据拖拽方向判断当前更新的状态
                    const refresher = this.state.direction == DIRECTION.DOWN ? this.refs.header : this.refs.bottom;
                    const enable = this.state.direction == DIRECTION.DOWN ? this.state.enableRefresh : this.state.enableLoadMore;

                    if(enable){
                        //更新子组件的状态
                        refresher.updateStatus(this._stateInShadow.status);
                    }

                }

            }
        });

        //定义isReach
        Object.defineProperty(this.state , 'isReach' , {
            get : () => {
                return this.state.reach[SIDE.TOP] || this.state.reach[SIDE.BOTTOM]
            }
        });

        //定义弹性
        Object.defineProperty(this.state , 'springConstant' , {
            value : mutants.env.os == mutants.env.constant.os.ios ? 1 : this.props.resistance
        });
    }

    //事件处理
    _eventHandler = {
        dragStart : (event) => {
            if(!this._checkInComponents(this.refs.domNode, event.target)){
                return;
            }

            const {pageY} = event.touches[0];
            this.state.deltaY = pageY;

            this.state.isPress = true;

            if(this.state.isReach){
                this.state.startDeltaY = pageY;
            }
        },

        /**
         * 安卓:
         *  使用touchmove来计算移动距离
         */
        dragMoveAndroid: (event) => {
            if(!this.state.isPress){
                return;
            }

            if(!this.state.isReach || this.state.status == STATUS.DOING){
                return;
            }

            const {pageY} = event.touches[0];
            this.state.deltaY = pageY;

            /**
             * reachTopFirstRunner:
             *      当屏幕滚动速度过快时, 两次touchmove事件之间的y轴高度差值较大,
             *      会导致当reachTop触发后,下一个touchmove的y轴产生跳屏的现象,
             *      所以当reachTop触发后的第一个touchmove事件才真正被认为是startDeltaY,
             *      从而解决跳屏问题;
             */
            if(this.state.reachFirstRunner){
                this.state.reachFirstRunner = false;
                this.state.startDeltaY = pageY;
            }

            const deltaY = (pageY - this.state.startDeltaY)/this.state.springConstant;
            // console.log('move deltaY', pageY, this.state.startDeltaY);

            this.state.direction = deltaY > 0 ? DIRECTION.DOWN : DIRECTION.UP;

            // console.log('deltaY' , deltaY);
            //如果没有到顶并且下拉了, 说明此时内容中又滚动条, 所以不进行移动
            if((!this.state.reach[SIDE.TOP] && deltaY > 0) || (!this.state.reach[SIDE.BOTTOM] && deltaY < 0)){
                return;
            }

            event.preventDefault();
            event.stopPropagation();

            this._scrollTo(0 , deltaY);

            if(Math.abs(deltaY) > this.props.distanceToRefresh){
                if(this.state.status != STATUS.TRIGGER){
                    this.state.status = STATUS.TRIGGER;
                }
            }else{
                if(this.state.status != STATUS.INITIAL){
                    this.state.status = STATUS.INITIAL;
                }
            }

        },

        /**
         * IOS:
         *  使用系统原生scroll事件来计算移动距离(设置内容的height超过100%来保证任何时候都有滚动条)
         */
        dragMoveIOS: () => {
            if(!this.state.isPress){
                return;
            }
            // console.log(this.state.scrollData);
            const {scrollTop , offsetHeight , scrollHeight} = this.state.scrollData;

            if(!this.state.isReach || this.state.status == STATUS.DOING){
                return;
            }

            let deltaY = undefined;
            //当已经到达内容顶部并继续往下拖拽时
            if(scrollTop < 0){
                deltaY = scrollTop/this.state.springConstant;
                this.state.direction = DIRECTION.DOWN;

                //当已经到达内容底部并继续往上拖拽时
            }else if(scrollTop + offsetHeight > scrollHeight){
                deltaY = (scrollTop + offsetHeight - scrollHeight)/this.state.springConstant;
                this.state.direction = DIRECTION.UP;


            }

            if(Math.abs(deltaY) > this.props.distanceToRefresh){
                if(this.state.status != STATUS.TRIGGER){
                    this.state.status = STATUS.TRIGGER;
                }
            }else{
                if(this.state.status != STATUS.INITIAL){
                    this.state.status = STATUS.INITIAL;
                }
            }
        },

        dragEnd : () => {
            if(!this.state.isPress){
                return;
            }

            this.state.isPress = false;

            if(this.state.status == STATUS.DOING){
                return;
            }

            const {distanceToRefresh , refreshHandler , moreHandler , loadingMinTime} = this.props;

            const triggerLine = {
                [DIRECTION.DOWN]: distanceToRefresh,
                [DIRECTION.UP]: 0 - distanceToRefresh
            };

            const handler = {
                [DIRECTION.DOWN]: refreshHandler,
                [DIRECTION.UP]: moreHandler
            };

            let targetDeltaY = 0;

            if(this.state.status == STATUS.TRIGGER){

                let actionHandler = undefined;
                let canDo = this.state.direction == DIRECTION.DOWN ? this.state.enableRefresh : this.state.enableLoadMore;

                if(canDo){
                    targetDeltaY = triggerLine[this.state.direction];
                    actionHandler = handler[this.state.direction];

                    this.state.status = STATUS.DOING;
                    setTimeout(()=>{
                        actionHandler(this._completeAnimation.bind(this));
                    },loadingMinTime);
                }


            }else{
                //ios下,弹性滚动自动还原
                // console.log('不刷新,还原');
            }

            this._scrollTo(0 , targetDeltaY , 400);

        },

        scroll : (event) => {
            // console.log('scrollHandler');
            this.state.scrollData = event;
            if(this.props.scrollHandler){
                this.props.scrollHandler(event);
            }
        },

        //滚动达到边缘的处理事件
        reach : (type) => {
            this.state.reach[type] = true;
            this.state.reachFirstRunner = true;

            this.state.startDeltaY = this.state.deltaY;
        },

        //滚动离开边缘的处理事件
        leave : (type) => {
            this.state.reach[type] = false;
            //如果是安卓,移动速度过快时,需要在leave时重置位置
            if(this.state.status != STATUS.DOING){
                this._scrollTo(0,0);
            }
        }
    };

    //提供对外滚动方法
    scrollTo(deltaY , duration=0){
        this.refs.scroller.scrollTo(deltaY, duration);
    }

    _scrollTo(deltaX = 0 , deltaY = 0 , duration = 0){
        const dom = this.refs.scrollerWrapper;

        //设置时间
        dom.style['-webkit-transition-duration'] = dom.style['transition-duration'] = duration + 'ms';
        //设置位移
        dom.style['-webkit-transform'] = dom.style['transform'] = `translate3d(${deltaX}px,${deltaY}px, 0px)`;

    }

    _completeAnimation(){
        //还原refresh状态
        if(this.state.status == STATUS.DOING){
            this.state.status = STATUS.INITIAL;
            this._scrollTo(0,0,400);
        }
    }

    //绑定事件
    _initEvent(){

        const {env} = mutants;
        //根据当前的环境判断处理方式
        const dragMoveHandler = env.os == env.constant.os.ios ? this._eventHandler.dragMoveIOS : this._eventHandler.dragMoveAndroid;

        // this.refs.domNode.addEventListener('touchstart' , this._eventHandler.dragStart , true);
        // this.refs.domNode.addEventListener('touchmove' , dragMoveHandler.bind(this) , true);
        // this.refs.domNode.addEventListener('touchend' , this._eventHandler.dragEnd , true);

        //改为使用window监听事件, 只有当事件在当前domNode内, 才会进行后续操作, 这样修改是为了兼容其他第三方类库, 比如客户管家中的侧拉;
        //不再在捕获阶段进行事件监听, 改为冒泡阶段, 防止阻止冒泡导致其他类库无法监听事件;
        window.addEventListener('touchstart' , this._eventHandler.dragStart);
        window.addEventListener('touchmove' , dragMoveHandler.bind(this));
        window.addEventListener('touchend' , this._eventHandler.dragEnd);
    };

    render(){
        const styles = this._getStyles();

        const refreshLoading = (
            <Refresher
                ref="header"
                style={{position:'absolute',top:0}}
                enable={this.state.enableRefresh}
                loadingMinTime={this.props.loadingMinTime}
                type={this.props.loadingStyle}
                distanceToRefresh={this.props.distanceToRefresh}
                statusLabel={this.props.refreshStatusLabel}
                direction={DIRECTION.DOWN}
                color={styles.refresh.color}
                status={this.state.status}
            />
        );


        const content = (
            <div ref="scrollerWrapper" style={styles.scrollerWrapper}>
                <ScrollerBase
                    ref="scroller"
                    onScroll={this._eventHandler.scroll}
                    onReachTop={this._eventHandler.reach.bind(this , SIDE.TOP)}
                    onLeaveTop={this._eventHandler.leave.bind(this , SIDE.TOP)}
                    onReachBottom={this._eventHandler.reach.bind(this , SIDE.BOTTOM)}
                    onLeaveBottom={this._eventHandler.leave.bind(this , SIDE.BOTTOM)}
                >
                    <div style={styles.contentWrapper}>
                        {this.props.children}
                    </div>
                </ScrollerBase>
            </div>
        );

        const moreLoading = (
            <Refresher
                ref="bottom"
                style={{position:'absolute',bottom:0}}
                enable={this.state.enableLoadMore}
                loadingMinTime={this.props.loadingMinTime}
                type={this.props.loadingStyle}
                distanceToRefresh={this.props.distanceToRefresh}
                statusLabel={this.props.moreStatusLabel}
                direction={DIRECTION.UP}
                color={styles.more.color}
                status={this.state.moreStatus}
            />
        );


        return (
            <div ref="domNode" style={styles.root}>
                {refreshLoading}

                {content}

                {moreLoading}
            </div>

        )


    }

}

export default WrapperForNative;