/**
 * Created by TonyJiang on 16/4/7.
 */
import React from 'react';
import ReactDOM from 'react-dom';

import {STATUS} from '../src/common/constant';
import Refresher from '../src/common/Refresher';
import LoadingIcon from '../src/common/LoadingIcon';
import ArrowIcon from '../src/common/ArrowIcon';

import 'should'
import chai from 'chai'
import sinon from 'sinon'
// import expect from 'expect'
import { mount, shallow } from 'enzyme';

const {assert, expect} = chai;

describe('Refresher' , ()=>{


    it('状态为初始状态时, 隐藏LoadingIcon' , () => {
        const wrapper = mount(<Refresher />);
        wrapper.instance().updateStatus(STATUS.INITIAL);

        const arrowIconDOM = ReactDOM.findDOMNode(wrapper.find(ArrowIcon).node);
        const loadingIconDOM = ReactDOM.findDOMNode(wrapper.find(LoadingIcon).node);

        expect(arrowIconDOM.style.display).to.equal('block');
        expect(loadingIconDOM.style.display).to.equal('none');
    });

    it('状态为触发状态时, 显示ArrowIcon, 并且旋转180度', () => {
        const wrapper = mount(<Refresher />);
        wrapper.instance().updateStatus(STATUS.TRIGGER);

        const arrowIconDOM = ReactDOM.findDOMNode(wrapper.find(ArrowIcon).node);
        const loadingIconDOM = ReactDOM.findDOMNode(wrapper.find(LoadingIcon).node);

        expect(arrowIconDOM.style.display).to.equal('block');
        expect(arrowIconDOM.style.transform).to.equal('rotateZ(180deg)');
        expect(loadingIconDOM.style.display).to.equal('none');

    });

    it('状态为Loading时, 显示LoadingIcon', () => {
        const wrapper = mount(<Refresher />);
        wrapper.instance().updateStatus(STATUS.DOING);

        const arrowIconDOM = ReactDOM.findDOMNode(wrapper.find(ArrowIcon).node);
        const loadingIconDOM = ReactDOM.findDOMNode(wrapper.find(LoadingIcon).node);

        expect(arrowIconDOM.style.display).to.equal('none');
        expect(loadingIconDOM.style.display).to.equal('block');
    });

    it('状态为完成状态时, 显示ArrowIcon' , () => {
        const wrapper = mount(<Refresher />);
        wrapper.instance().updateStatus(STATUS.DONE);

        const arrowIconDOM = ReactDOM.findDOMNode(wrapper.find(ArrowIcon).node);
        const loadingIconDOM = ReactDOM.findDOMNode(wrapper.find(LoadingIcon).node);

        expect(arrowIconDOM.style.display).to.equal('');
        expect(loadingIconDOM.style.display).to.equal('none');
    });


});
