/**
 * Created by TonyJiang on 16/12/19.
 */
import React from 'react';
import ReactDOM from 'react-dom';

import {STATUS, DIRECTION, SIDE} from '../src/common/constant';
import Refresher from '../src/common/Refresher';
import ScrollBar from '../src/common/ScrollBar';
import ScrollerBase from '../src/WrapperForNative/ScrollerBase';
import WrapperForNative from '../src/WrapperForNative';

import 'should'
import chai from 'chai'
import sinon from 'sinon'
import { mount, shallow } from 'enzyme';
const {assert, expect} = chai;

const initSimulateEvent = (target, deltaY) => {
    return {
        target:target,
        touches: [
            {
                pageY: deltaY
            }
        ],
        preventDefault: () => {},
        stopPropagation: () => {}
    }
}


describe('WrapperForNative 测试', () => {
    describe('组件结构' , () => {

        it('应该包含HeaderRefresher, FooterRefresher, ScrollerBase和传入的子组件', () => {

            const ChildComponent = () => <div>ChildComponent</div>;

            const wrapper = shallow(
                <WrapperForNative >
                    <ChildComponent/>
                </WrapperForNative>
            );

            expect(wrapper.find(Refresher)).to.have.length(2);
            expect(wrapper.find(ScrollerBase)).to.have.length(1);
            expect(wrapper.find(ChildComponent)).to.have.length(1);


        });

    });

    describe('事件处理' , () => {

        describe('安卓环境', () => {
            const distanceToRefresh = 40;
            const refreshSPY = sinon.spy();
            const wrapper = mount(
                <WrapperForNative refreshHandler={refreshSPY} distanceToRefresh={distanceToRefresh} resistance={1}>
                    <div style={{height: 10000}}></div>
                </WrapperForNative>
            );

            const instance = wrapper.instance();

            afterEach(() => {
                refreshSPY.reset();
            });


            it('手指按下, 组件内部状态: press=true' , () => {
                instance._eventHandler.reach(SIDE.TOP);
                instance._eventHandler.dragStart(initSimulateEvent(ReactDOM.findDOMNode(instance), 0));
                // console.log(wrapper.node);
                // instance._eventHandler.dragStart(wrapper.node, 100);

                expect(instance.state.isPress).to.equal(true);


            });

            it('第一次move更新startDeltaY, 不做其他处理', (done) => {
                // instance._eventHandler.reach(SIDE.TOP);
                const beforeStatus = instance.state.reachFirstRunner;
                instance._eventHandler.dragMoveAndroid(initSimulateEvent(ReactDOM.findDOMNode(instance), 2));
                const afterStatus = instance.state.reachFirstRunner;

                //等待requestAnimationFrame执行
                setTimeout(() => {
                    expect(beforeStatus).to.equal(true);
                    expect(afterStatus).to.equal(false);
                    expect(instance.state.startDeltaY).to.equal(2);

                    done();

                }, 50);

            });

            it('手指向下移动10px, 组件内部状态: direction=down, reachTop=true', (done) => {
                instance._eventHandler.dragMoveAndroid(initSimulateEvent(ReactDOM.findDOMNode(instance), 2 + 10));

                //等待requestAnimationFrame执行
                setTimeout(() => {
                    expect(instance.state.direction).to.equal(DIRECTION.DOWN);

                    done();

                }, 50);

            });

            it('手指向下移动超过触发线, 组件内部状态: direction=down, reachTop=true', (done) => {
                instance._eventHandler.dragMoveAndroid(initSimulateEvent(ReactDOM.findDOMNode(instance), 2 + distanceToRefresh + 1));

                //等待requestAnimationFrame执行
                setTimeout(() => {
                    expect(instance.state.status).to.equal(STATUS.TRIGGER);

                    done();

                }, 50);

            });

            it('松开手指, 组件内部状态: direction=down, reachTop=true, status=loading, refreshHandler被执行', (done) => {
                instance._eventHandler.dragEnd();

                setTimeout(() => {

                    expect(instance.state.direction).to.equal(DIRECTION.DOWN);
                    expect(instance.state.reach[SIDE.TOP]).to.equal(true);
                    expect(instance.state.status).to.equal(STATUS.DOING);
                    expect(refreshSPY.calledOnce);


                    done();

                });


            });

        });

        describe('苹果环境', () => {
            const distanceToRefresh = 40;
            const refreshSPY = sinon.spy();
            const wrapper = mount(
                <WrapperForNative refreshHandler={refreshSPY} distanceToRefresh={distanceToRefresh} resistance={1}>
                    <div style={{height: 10000}}></div>
                </WrapperForNative>
            );

            const instance = wrapper.instance();

            afterEach(() => {
                refreshSPY.reset();
            });


            it('手指按下, 组件内部状态: press=true' , () => {
                instance._eventHandler.reach(SIDE.TOP);
                instance._eventHandler.dragStart(initSimulateEvent(ReactDOM.findDOMNode(instance), 0));
                // console.log(wrapper.node);
                // instance._eventHandler.dragStart(wrapper.node, 100);

                expect(instance.state.isPress).to.equal(true);


            });

            it('页面向下滚动10px, 组件内部状态: direction=down, reachTop=true', (done) => {
                instance.state.scrollData = {
                    scrollTop: -10,
                    offsetHeight: 500,
                    scrollHeight: 10000
                };
                instance._eventHandler.dragMoveIOS();

                //等待requestAnimationFrame执行
                setTimeout(() => {
                    expect(instance.state.direction).to.equal(DIRECTION.DOWN);

                    done();

                }, 50);

            });


            it('页面继续向下滚动超过触发线, 组件内部状态: direction=down, reachTop=true, status=trigger', (done) => {
                instance.state.scrollData = {
                    scrollTop: -41,
                    offsetHeight: 500,
                    scrollHeight: 10000
                };
                instance._eventHandler.dragMoveIOS();

                //等待requestAnimationFrame执行
                setTimeout(() => {
                    expect(instance.state.direction).to.equal(DIRECTION.DOWN);
                    expect(instance.state.status).to.equal(STATUS.TRIGGER);

                    done();

                }, 50);

            });

            it('松开手指, 组件内部状态: direction=down, reachTop=true, status=loading, refreshHandler被执行', (done) => {
                instance._eventHandler.dragEnd();

                setTimeout(() => {

                    expect(instance.state.direction).to.equal(DIRECTION.DOWN);
                    expect(instance.state.reach[SIDE.TOP]).to.equal(true);
                    expect(instance.state.status).to.equal(STATUS.DOING);
                    expect(refreshSPY.calledOnce);


                    done();

                });


            });




        });








        // it('手指继续向上移动20px, 组件内部状态: direction=top, reachTop=false', (done) => {
        //     instance._eventHandler.dragMove(initSimulateEvent(ReactDOM.findDOMNode(instance), 110 - 20));
        //
        //     //等待requestAnimationFrame执行
        //     setTimeout(() => {
        //         expect(instance.state.direction).to.equal(DIRECTION.UP);
        //         expect(instance.state.reach[SIDE.TOP]).to.equal(false);
        //
        //         done();
        //     }, 50);
        // });
        //
        // it('手指继续向下移动200px, 组件内部状态: direction=top, reachTop=true, status=trigger, refreshSide=top', (done) => {
        //     instance._eventHandler.dragMove(initSimulateEvent(ReactDOM.findDOMNode(instance), 90 + 200));
        //
        //     //等待requestAnimationFrame执行
        //     setTimeout(() => {
        //         instance._eventHandler.dragMove(initSimulateEvent(ReactDOM.findDOMNode(instance), 90 + 200 + 1));
        //
        //         expect(instance.state.direction).to.equal(DIRECTION.DOWN);
        //         expect(instance.state.reach[SIDE.TOP]).to.equal(true);
        //         expect(instance.state.status).to.equal(STATUS.TRIGGER);
        //         expect(instance.state.refreshSide).to.equal(SIDE.TOP);
        //
        //         done();
        //     }, 50);
        //
        // });
        //
        // it('松开手指, 组件内部状态: direction=top, reachTop=true, status=loading, refreshHandler被执行', (done) => {
        //     instance._eventHandler.dragEnd();
        //
        //     setTimeout(() => {
        //
        //         expect(instance.state.direction).to.equal(DIRECTION.DOWN);
        //         expect(instance.state.reach[SIDE.TOP]).to.equal(true);
        //         expect(instance.state.status).to.equal(STATUS.DOING);
        //         expect(refreshSPY.calledOnce);
        //
        //
        //         done();
        //
        //     });
        //
        //
        // });


    });
});