/**
 * Created by TonyJiang on 16/12/16.
 */
import React from 'react';
import ReactDOM from 'react-dom';

import {STATUS, DIRECTION, SIDE} from '../src/common/constant';
import Refresher from '../src/common/Refresher';
import ScrollBar from '../src/common/ScrollBar';
import WrapperForSimulate from '../src/WrapperForSimulate';

import 'should'
import chai from 'chai'
import sinon from 'sinon'
import { mount, shallow } from 'enzyme';
const {assert, expect} = chai;

const initSimulateEvent = (target, deltaY) => {
    return {
        target:target,
        touches: [
            {
                pageY: deltaY
            }
        ],
        preventDefault: () => {}
    }
}


describe('WrapperForSimulate 测试' , () => {

    describe('组件结构' , () => {

        it('应该包含HeaderRefresher, FooterRefresher, ScrollBar和传入的子组件', () => {

            const ChildComponent = () => <div>ChildComponent</div>;

            const wrapper = shallow(
                <WrapperForSimulate >
                    <ChildComponent/>
                </WrapperForSimulate>
            );

            expect(wrapper.find(Refresher)).to.have.length(2);
            expect(wrapper.find(ScrollBar)).to.have.length(1);
            expect(wrapper.find(ChildComponent)).to.have.length(1);


        });

    });

    describe('事件处理' , () => {

        const refreshSPY = sinon.spy();
        const wrapper = mount(
            <WrapperForSimulate refreshHandler={refreshSPY}>
                <div style={{height: 10000}}></div>
            </WrapperForSimulate>
        );

        const instance = wrapper.instance();

        afterEach(() => {
            refreshSPY.reset();
        });


        it('手指按下, 组件内部状态: press=true' , () => {

            instance._eventHandler.dragStart(initSimulateEvent(ReactDOM.findDOMNode(instance), 100));
            // console.log(wrapper.node);
            // instance._eventHandler.dragStart(wrapper.node, 100);

            expect(instance._state.press).to.equal(true);


        });

        it('手指向下移动10px, 应该调用scroll方法, 组件内部状态: direction=down, reachTop=true', (done) => {
            instance._eventHandler.dragMove(initSimulateEvent(ReactDOM.findDOMNode(instance), 100 + 10));

            //等待requestAnimationFrame执行
            setTimeout(() => {
                expect(instance.state.direction).to.equal(DIRECTION.DOWN);
                expect(instance.state.reach[SIDE.TOP]).to.equal(true);

                done();

            }, 50);

        });

        it('手指继续向上移动20px, 组件内部状态: direction=top, reachTop=false', (done) => {
            instance._eventHandler.dragMove(initSimulateEvent(ReactDOM.findDOMNode(instance), 110 - 20));

            //等待requestAnimationFrame执行
            setTimeout(() => {
                expect(instance.state.direction).to.equal(DIRECTION.UP);
                expect(instance.state.reach[SIDE.TOP]).to.equal(false);

                done();
            }, 50);
        });

        it('手指继续向下移动200px, 组件内部状态: direction=top, reachTop=true, status=trigger, refreshSide=top', (done) => {
            instance._eventHandler.dragMove(initSimulateEvent(ReactDOM.findDOMNode(instance), 90 + 200));

            //等待requestAnimationFrame执行
            setTimeout(() => {
                instance._eventHandler.dragMove(initSimulateEvent(ReactDOM.findDOMNode(instance), 90 + 200 + 1));

                expect(instance.state.direction).to.equal(DIRECTION.DOWN);
                expect(instance.state.reach[SIDE.TOP]).to.equal(true);
                expect(instance.state.status).to.equal(STATUS.TRIGGER);
                expect(instance.state.refreshSide).to.equal(SIDE.TOP);

                done();
            }, 50);

        });

        it('松开手指, 组件内部状态: direction=top, reachTop=true, status=loading, refreshHandler被执行', (done) => {
           instance._eventHandler.dragEnd();

           setTimeout(() => {

               expect(instance.state.direction).to.equal(DIRECTION.DOWN);
               expect(instance.state.reach[SIDE.TOP]).to.equal(true);
               expect(instance.state.status).to.equal(STATUS.DOING);
               expect(refreshSPY.calledOnce);


               done();

           });


        });


    });

    // describe('内容高度动态监测', () => {
    //
    //     const ContentComponent = () => <div style={{height: 10000}}></div>;
    //
    //     const wrapper = mount(
    //         <WrapperForSimulate displayWidth={200} displayHeight={1000}>
    //             <ContentComponent />
    //         </WrapperForSimulate>
    //     );
    //
    //     const instance = wrapper.instance();
    //
    //     it('滚动到距离底部100位置, 然后调整高度减少200px, 组件内部状态: reactBottom=true' , (done) => {
    //
    //         instance.scrollTo(9900);
    //
    //         const contentDOM = ReactDOM.findDOMNode(wrapper.find(ContentComponent).node);
    //         contentDOM.style.height = 8000;
    //
    //         setTimeout(() => {
    //             expect(instance.state.reach[SIDE.BOTTOM]).to.equal(true);
    //
    //             done();
    //
    //
    //         }, 100);
    //
    //
    //
    //     });
    //
    //
    // });


});